import ipaddress
from typing import Optional, List

import pydantic

from hivepy.field_management.enums import FieldType
from hivepy.field_management.models import BaseField


class Field(BaseField):
    """Field class."""
    type: None = pydantic.Field(default=None, exclude=True)

    @pydantic.field_validator('type', mode='before')
    def clear_type(cls, v: Optional[FieldType]) -> None:
        """Clear type."""
        v = None
        return v


class Text(Field):
    """Text field."""
    ...


class SingleTextSuggested(Field):
    """Text field with single suggested value."""
    ...


class MultiTextSuggested(Field):
    """Text field with suggested values."""
    ...


class Checkbox(Field):
    """Single checkbox field."""
    ...


class Checkboxes(Field):
    """Checkboxes field."""
    ...


class RadioButton(Field):
    """RadioButtons field."""
    ...


class Select(Field):
    """Select field."""
    ...


class MultiSelect(Field):
    """MultiSelect field."""
    ...


class Link(Field):
    """Link field."""
    ...


class TextMarkdown(Field):
    """Text field with markdown."""
    ...


class Image(Field):
    """Image field."""
    ...


class File(Field):
    """File field."""
    ...


class Float(Field):
    """Float field."""
    ...


class Integer(Field):
    """Integer field."""
    ...


class Date(Field):
    """Date field."""
    ...


class DateTime(Field):
    """DateTime field."""
    ...


class UUID(Field):
    """UUID field."""
    ...


class Ip(Field):
    """IP field."""
    value: Optional[List[ipaddress.IPv4Address]] = pydantic.Field(default_factory=list)


class Hostname(Field):
    """Hostname field."""
    ...


class Asset(Field):
    """Asset field."""
    ...


class Request(Field):
    """Request field."""
    ...


class Datasource(Field):
    """Datasource field."""
    ...


class Status(Field):
    """Status field."""
    ...


class CVSSBaseScore(Field):
    """CVSS Base Score field."""
    ...


class CVSSBaseVector(Field):
    """CVSS Base Vector field."""
    ...


FIELD_TYPE_MAP = {
    FieldType.INTEGER: Integer,
    FieldType.FLOAT: Float,
    FieldType.TEXT: Text,
    FieldType.SINGLE_TEXT_SUGGESTED: SingleTextSuggested,
    FieldType.MULTI_TEXT_SUGGESTED: MultiTextSuggested,
    FieldType.CHECKBOX: Checkbox,
    FieldType.CHECKBOXES: Checkboxes,
    FieldType.TEXT_MARKDOWN: TextMarkdown,
    FieldType.RADIOBUTTON: RadioButton,
    FieldType.SELECT: Select,
    FieldType.MULTISELECT: MultiSelect,
    FieldType.LINK: Link,
    FieldType.IMAGE: Image,
    FieldType.FILE: File,
    FieldType.DATE: Date,
    FieldType.DATETIME: DateTime,
    FieldType.UUID: UUID,
    FieldType.IP: Ip,
    FieldType.HOSTNAME: Hostname,
    FieldType.ASSET: Asset,
    FieldType.REQUEST: Request,
    FieldType.DATASOURCE: Datasource,
    FieldType.STATUS: Status,
    FieldType.CVSS_SCORE: CVSSBaseScore,
    FieldType.CVSS_VECTOR: CVSSBaseVector,
}



from typing import Union, ForwardRef

from hivepy.schema_management.factories.schema_factory import SchemaFactory
from hivepy.template_management.builders.project_template_builder import ProjectTemplateBuilder
from hivepy.template_management.builders.issue_template_builder import IssueTemplateBuilder


ProjectTemplate = ForwardRef('ProjectTemplate')
IssueTemplate = ForwardRef('IssueTemplate')


class TemplateBuilder:
    """Template builder class."""
    def __init__(self) -> None:
        """Initialize the template builder with schema and template builders."""
        self.schema_factory: SchemaFactory = SchemaFactory()
        self.project_template_builder: ProjectTemplateBuilder = ProjectTemplateBuilder(self.schema_factory)
        self.issue_template_builder: IssueTemplateBuilder = IssueTemplateBuilder(self.schema_factory)

    def build(self, template_type: str, template_data) -> Union[ProjectTemplate, IssueTemplate]:
        """Build a template object from a template data dictionary"""
        if template_type == 'project':
            return self.project_template_builder.build(template_data)
        elif template_type == 'issue':
            return self.issue_template_builder.build(template_data)
        else:
            raise ValueError(f'Unknown template type: {template_type}')

    def build_project_template(self, template_data) -> ProjectTemplate:
        """Build a project template object from a template data dictionary"""
        return self.project_template_builder.build(template_data)

    def build_issue_template(self, template_data) -> IssueTemplate:
        """Build an issue template object from a template data dictionary"""
        return self.issue_template_builder.build(template_data)



from datetime import datetime
from typing import Dict, Optional, Any, List

import pydantic


class Project(pydantic.BaseModel):
    """Project model."""
    id: str = pydantic.Field(alias='projectId')
    name: str = pydantic.Field(alias='projectName')
    description: Optional[str] = pydantic.Field(default=None, alias='projectDescription')

    scope: Optional[str] = pydantic.Field(default=None, alias='projectScope')
    permissions: Optional[Dict] = pydantic.Field(default=None, alias='projectPermissions')

    is_archived: bool = pydantic.Field(default=None, alias='projectIsArchived')
    archive_date: Optional[str] = pydantic.Field(default=None, alias='projectArchiveDate')

    create_date: Optional[str] = pydantic.Field(default=None, alias='projectCreateDate')
    update_date: Optional[str] = pydantic.Field(default=None, alias='projectUpdateDate')
    start_date: Optional[str] = pydantic.Field(default=None, alias='projectStartDate')
    end_date: Optional[str] = pydantic.Field(default=None, alias='projectEndDate')
    issue_settings: Optional[Dict] = pydantic.Field(default=None, alias='issueSettings')

    users: Optional[List[Dict]] = pydantic.Field(default=None, alias='projectUsers')

    application_connect: Optional[str] = pydantic.Field(default=None, alias='applicationConnect', repr=False)
    connection_id: Optional[str] = pydantic.Field(default=None, alias='projectConnectionId', repr=False)
    connection_name: Optional[str] = pydantic.Field(default=None, alias='connectionName', repr=False)
    last_incoming_pong: Optional[str] = pydantic.Field(default=None, alias='lastIncomingPong', repr=False)
    last_outgoing_ping: Optional[str] = pydantic.Field(default=None, alias='lastOutgoingPing', repr=False)
    connection: Optional[Any] = pydantic.Field(default=None, alias='hive', repr=False, exclude=True)

    model_config = pydantic.ConfigDict(
        populate_by_name=True
    )

    @pydantic.field_validator('create_date', 'update_date', 'start_date', 'end_date', mode='before')
    def validate_datetime(cls, value: str) -> str:
        """Validate datetime field."""
        try:
            dt: datetime = datetime.strptime(value, '%Y-%m-%dT%H:%M:%S.%fZ')
        except ValueError:
            raise ValueError('Invalid datetime.')
        return datetime.strftime(dt, '%d-%m-%Y %H:%M:%S')

    def __str__(self) -> str:
        """Return string representation of object."""
        return f'{self.__class__.__name__}({"".join(f"{key}={value}, " for key, value in self.model_dump().items())})'



from typing import Dict, ForwardRef, Callable

from hivepy.schema_management.builders.schema_builder import SchemaBuilder

Schema = ForwardRef('Schema')


def caching(func) -> Callable:
    """Caching decorator."""
    def wrapper(self, schema_type: str, schema: Dict) -> Schema:
        """Wrapper."""
        schema_id_version: str = f"{schema.get('id')}_{schema.get('version')}"
        if schema_id_version not in self.cache:
            self.cache[schema_id_version] = func(self, schema_type, schema)
        return self.cache[schema_id_version]
    return wrapper


class SchemaFactory:
    """Schema factory class."""
    def __init__(self):
        self.cache: Dict = {}  # {'schema_id_version': 'schema': Schema}
        self.schema_builder: SchemaBuilder = SchemaBuilder()

    @caching
    def make_schema(self, schema_type: str, schema: Dict) -> Schema:
        """Make schema."""
        return self.schema_builder(schema_type, schema)

    def make_issue_schema(self, schema: Dict) -> Schema:
        """Make issue schema."""
        return self.schema_builder('issue', schema)

    def make_project_schema(self, schema: Dict) -> Schema:
        """Make project schema."""
        return self.schema_builder('project', schema)

    def __call__(self, schema_type: str, schema: Dict) -> Schema:
        """Call."""
        return self.make_schema(schema_type, schema)



import uuid
from typing import Optional, Dict, Self, List, Union

from hivepy.api_management.deserializator import ModeHandler
from hivepy.api_management.enums import ClientMode, ObjectType
from hivepy.api_management.object_builder import Builder as ObjectBuilder
from hivepy.issue_management.issue import Issue
from hivepy.project_management.project import Project
from hivepy.rest import State, exceptions, HTTPClient


@ModeHandler
class HiveApi:
    def __init__(self, mode=ClientMode.OBJECT, **kwargs) -> None:
        """Initialize HiveApi."""
        self.state: State = State.NOT_CONNECTED
        self.base_url: Optional[str] = None

        self.http_client: HTTPClient = HTTPClient()
        self.builder: ObjectBuilder = ObjectBuilder()
        self.mode: ClientMode = mode

        if kwargs:
            self.connect(**kwargs)

    def authenticate(self, username: str, password: str) -> None:
        """Authenticate in Hive."""
        response = self.http_client.session.post(f"{self.base_url}/session", json={
            'userLogin': username,
            'userPassword': password,
        })

        try:
            cookie = response.cookies.get('BSESSIONID')
            self.http_client.add_headers({'Cookie': f'BSESSIONID={cookie}'})
            self.state = State.CONNECTED
        except AttributeError:
            raise exceptions.RestConnectionError('Could not get authentication cookie. Something wrong with credentials'
                                                 'or server.')

    @staticmethod
    def create_base_url(url: str, port: Optional[int] = None) -> str:
        """Create base URL from server and port."""
        proto, hostname, *str_port = url.split(':')

        if not proto:
            raise exceptions.ServerUrlNotDefinedError('Protocol not defined in server URL.')

        if str_port:
            port = int(str_port[0])
            url = f'{proto}://{hostname}'

        if url.startswith('https') and not port:
            port = 443
        elif url.startswith('http') and not port:
            port = 80

        return f'{url.strip("/")}:{port}/api'

    def connect(self,
                url: str,
                username: str,
                password: str,
                port: Optional[int] = None,
                verify: bool = True,
                proxies: Optional[Dict] = None,
                proxy: Optional[str] = None,
                mode: Optional[ClientMode] = None,
                **kwargs,
                ) -> Self:
        """Connect to Hive."""
        self.base_url = self.create_base_url(url, port)
        if mode:
            self.mode = mode
        if proxies:
            self.http_client.update_params(proxies=proxies)
        if proxy:
            self.http_client.update_params(proxies={'http': proxy, 'https': proxy})
        if verify is False:
            self.http_client.update_params(verify=False)
        if cert := kwargs.get('cert'):
            self.http_client.update_params(cert=cert)
        self.authenticate(username, password)
        return self

    def get_session(self) -> Dict:
        """Get session information."""
        response = self.http_client.get(f'{self.base_url}/session')
        return response

    def get_licence(self) -> Dict:
        """Get licence information."""
        response = self.http_client.get(f'{self.base_url}/licence')
        return response

    def get_users(self) -> List[Dict]:
        """Get all users."""
        response = self.http_client.get(f'{self.base_url}/user')
        return response

    def get_groups(self) -> Dict:
        """Get all groups."""
        response = self.http_client.get(f'{self.base_url}/groups')
        return response

    @ModeHandler.deserialize(to=ObjectType.PROJECT)
    def get_projects(self, **params) -> List[Project]:
        """Get all projects."""
        return self.http_client.post(f'{self.base_url}/project/filter', params=params)

    @ModeHandler.deserialize(to=ObjectType.PROJECT)
    def get_project(self, project_id: Union[str, uuid.UUID]) -> Dict:
        """Get project by ID."""
        return self.http_client.get(f'{self.base_url}/project/{project_id}')

    def get_project_templates(self, **params) -> Dict:
        """Get all project templates."""
        response = self.http_client.get(f'{self.base_url}/project/templates', params=params)
        return response

    def get_project_custom_fields(self) -> List[Dict]:
        """Get all project custom fields."""
        response = self.http_client.get(f'{self.base_url}/project/schema/accessible/fields')
        return response

    def get_issues(self, project_id: str, offset: int = 0, limit: int = 100) -> List[Issue]:
        response = self.http_client.post(
            url=f'{self.base_url}/project/{project_id}/graph/issue_list?offset={offset}&limit={limit}',
            json={})
        return [Issue(**x) for x in response.get('items', [])]

    def get_file(self, project_id: str, file_id: str) -> bytes:
        return self.http_client.get(f'{self.base_url}/project/{project_id}/graph/file/{file_id}')

    def get_issues_schemas(self) -> List[Dict]:
        """Get all issues schemes."""
        response = self.http_client.get(f'{self.base_url}/customization/issues')
        return response

    def get_project_issue_scheme(self, project_id: Union[str, uuid.UUID]) -> Dict:
        """Get project issue scheme."""
        response = self.http_client.get(f'{self.base_url}/project/{project_id}/settings/issues')
        return response

    def get_project_issue(self, project_id: Union[str, uuid.UUID], issue_id: Union[str, uuid.UUID]) -> Dict:
        """Get project issue."""
        response = self.http_client.get(f'{self.base_url}/project/{project_id}/graph/issues/{issue_id}')
        return response

    def get_project_tasks(self, project_id: Union[str, uuid.UUID], **params) -> Dict:
        """Get project tasks."""
        response = self.http_client.post(f'{self.base_url}/project/{project_id}/tasks', params=params)
        return response

    def get_project_issue_sources(self, project_id: Union[str, uuid.UUID]) -> List[Dict]:
        """Get project issue sources."""
        response = self.http_client.get(f'{self.base_url}/project/{project_id}/graph/issues/sources')
        return response

    def get_project_issue_statuses(self, project_id: Union[str, uuid.UUID]) -> List[Dict]:
        """Get project issue statuses."""
        response = self.http_client.get(f'{self.base_url}/project/{project_id}/settings/issues/statuses')
        return response

    def get_project_checklists(self, project_id: Union[str, uuid.UUID]) -> List[Dict]:
        """Get project checklists."""
        response = self.http_client.get(f'{self.base_url}/project/{project_id}/graph/checklist')
        return response

    def get_project_apps(self, project_id: Union[str, uuid.UUID]) -> List[Dict]:
        """Get project apps."""
        response = self.http_client.get(f'{self.base_url}/project/{project_id}/graph/apps')
        return response

    def get_project_credentials(self, project_id: Union[str, uuid.UUID]) -> List[Dict]:
        """Get project credentials."""
        response = self.http_client.get(f'{self.base_url}/project/{project_id}/graph/nodes/credentials ')
        return response

    def get_project_description(self, project_id: Union[str, uuid.UUID]) -> Dict:
        """Get project description."""
        response = self.http_client.get(f'{self.base_url}/project/{project_id}/description')
        return response

    def get_checklist_templates(self) -> List[Dict]:
        """Get all checklist templates."""
        response = self.http_client.get(f'{self.base_url}/templates/checklists')
        return response

    def get_issues_templates(self) -> List[Dict]:
        """Get all issues templates."""
        response = self.http_client.get(f'{self.base_url}/templates/issues')
        return response



from hivepy.template_management.models.base_template import BaseTemplate
from hivepy.schema_management.models.issue_schema import IssueSchema


class IssueTemplate(BaseTemplate):
    """Issue template model."""
    schema: IssueSchema
    ...



from typing import Dict

from hivepy.schema_management.factories.schema_factory import SchemaFactory
from hivepy.template_management.models.project_template import ProjectTemplate


class ProjectTemplateBuilder:
    """Project template builder. """

    def __init__(self, schema_factory) -> None:
        """Initiate project template builder."""
        self.schema_factory: SchemaFactory = schema_factory

    def build(self, project_data: Dict) -> ProjectTemplate:
        """Build project template."""
        project = project_data.copy()
        if schema := project.pop('projectSchema', {}):
            project['schema'] = self.schema_factory.make_project_schema(schema)
        return ProjectTemplate(**project)

    def __call__(self, project: Dict) -> ProjectTemplate:
        """Call. """
        return self.build(project)



class ClientError(Exception):
    """Base exception for Client."""
    ...


class ClientConnectionError(ClientError):
    """Exception for connection errors."""
    def __init__(self, message):
        """Initialize ClientConnectionError."""
        super().__init__(message)


class SocksProxyError(ClientConnectionError):
    """Exception for connection errors."""
    def __init__(self, message="Couldn't connect via provided socks proxy. Check it."):
        """Initialize ClientConnectionError."""
        super().__init__(message)



from hivepy.schema_management import builders, factories, models
from hivepy.schema_management.builders.schema_builder import SchemaBuilder
from hivepy.schema_management.builders.project_schema_builder import ProjectSchemaBuilder
from hivepy.schema_management.factories.schema_factory import SchemaFactory


__all__ = [
    "builders",
    "factories",
    "models",
    'SchemaFactory',
]



from typing import Any, Optional, Dict, ForwardRef

import pydantic

from hivepy.field_management.enums import BaseFieldType, FieldType, ValueType
from hivepy.field_management import FIELD_TYPE_MAP


class FieldModelBuilder:
    @classmethod
    def define_field_type(cls, base_field: Dict) -> FieldType:
        if cls.is_single_suggested_text(base_field):
            return FieldType.SINGLE_TEXT_SUGGESTED
        if cls.is_multi_suggested_text(base_field):
            return FieldType.MULTI_TEXT_SUGGESTED
        if cls.is_text(base_field):
            return FieldType.TEXT
        if cls.is_markdown(base_field):
            return FieldType.TEXT_MARKDOWN
        if cls.is_radiobutton(base_field):
            return FieldType.RADIOBUTTON
        if cls.is_checkbox(base_field):
            return FieldType.CHECKBOX
        if cls.is_checkboxes(base_field):
            return FieldType.CHECKBOXES
        if cls.is_select(base_field):
            return FieldType.SELECT
        if cls.is_multiselect(base_field):
            return FieldType.MULTISELECT
        if cls.is_date(base_field):
            return FieldType.DATE
        if cls.is_datetime(base_field):
            return FieldType.DATETIME
        if cls.is_link(base_field):
            return FieldType.LINK
        if cls.is_image(base_field):
            return FieldType.IMAGE
        if cls.is_file(base_field):
            return FieldType.FILE
        if cls.is_integer(base_field):
            return FieldType.INTEGER
        if cls.is_float(base_field):
            return FieldType.FLOAT
        if cls.is_cvss_base_score(base_field):
            return FieldType.CVSS_SCORE
        if cls.is_cvss_base_vector(base_field):
            return FieldType.CVSS_VECTOR
        if cls.is_asset(base_field):
            return FieldType.ASSET
        if cls.is_request(base_field):
            return FieldType.REQUEST
        if cls.is_datasource(base_field):
            return FieldType.DATASOURCE
        if cls.is_status(base_field):
            return FieldType.STATUS
        if cls.is_hostname(base_field):
            return FieldType.HOSTNAME
        if cls.is_uuid(base_field):
            return FieldType.UUID
        if cls.is_ip(base_field):
            return FieldType.IP
        raise ValueError(f"Unknown field type: {base_field['type']}\n{base_field}")

    @classmethod
    def is_text(cls, base_field: Dict) -> bool:
        """Check if field is text field."""
        return base_field['type'] == BaseFieldType.TEXT and not base_field.get('allowed_values')

    @classmethod
    def is_single_suggested_text(cls, base_field: Dict) -> bool:
        """Check if field is single suggested text field."""
        return base_field['type'] == BaseFieldType.TEXT_SUGGESTED and base_field.get('is_list') is False

    @classmethod
    def is_multi_suggested_text(cls, base_field: Dict) -> bool:
        """Check if field is multi suggested text field."""
        return base_field['type'] == BaseFieldType.TEXT_SUGGESTED and base_field.get('is_list') is True

    @classmethod
    def is_radiobutton(cls, base_field: Dict) -> bool:
        """Check if field is radiobutton field."""
        return base_field['type'] == BaseFieldType.TEXT \
            and base_field.get('allowed_values') \
            and base_field.get('is_list') is False \
            and base_field.get('select') is False

    @classmethod
    def is_checkbox(cls, base_field: Dict) -> bool:
        """Check if field is single checkbox field."""
        return base_field['type'] == BaseFieldType.BOOLEAN \
            and not base_field.get('allowed_values') \
            and not base_field.get('is_list', False) \
            and not base_field.get('select', False)

    @classmethod
    def is_checkboxes(cls, base_field: Dict) -> bool:
        """Check if field is checkboxes field."""
        return base_field['type'] == BaseFieldType.TEXT \
            and base_field.get('allowed_values') \
            and base_field.get('is_list', False) \
            and not base_field.get('select', False)

    @classmethod
    def is_select(cls, base_field: Dict) -> bool:
        """Check if field is select field."""
        return (base_field['type'] == BaseFieldType.TEXT
                and base_field.get('allowed_values')
                and not base_field.get('is_list')
                and base_field.get('select'))

    @classmethod
    def is_multiselect(cls, base_field: Dict) -> bool:
        """Check if field is multi select field."""
        return base_field['type'] == BaseFieldType.TEXT \
            and base_field.get('allowed_values') \
            and base_field.get('is_list') \
            and base_field.get('select')

    @classmethod
    def is_date(cls, base_field: Dict) -> bool:
        """Check if field is date field."""
        return base_field['type'] == BaseFieldType.DATE

    @classmethod
    def is_datetime(cls, base_field: Dict) -> bool:
        """Check if field is datetime field."""
        return base_field['type'] == BaseFieldType.DATETIME

    @classmethod
    def is_link(cls, base_field: Dict) -> bool:
        """Check if field is link field."""
        return base_field['type'] == BaseFieldType.LINK

    @classmethod
    def is_image(cls, base_field: Dict) -> bool:
        """Check if field is image field."""
        return base_field['type'] == BaseFieldType.IMAGE

    @classmethod
    def is_file(cls, base_field: Dict) -> bool:
        """Check if field is file field."""
        return base_field['type'] == BaseFieldType.FILE

    @classmethod
    def is_integer(cls, base_field: Dict) -> bool:
        """Check if field is integer field."""
        return base_field['type'] == BaseFieldType.INTEGER

    @classmethod
    def is_markdown(cls, base_field: Dict) -> bool:
        """Check if field is markdown field."""
        return base_field['type'] == BaseFieldType.TEXT_MARKDOWN

    @classmethod
    def is_float(cls, base_field: Dict) -> bool:
        """Check if field is float field."""
        return base_field['type'] == BaseFieldType.FLOAT

    @classmethod
    def is_cvss_base_score(cls, base_field: Dict) -> bool:
        """Check if field is CVSS Base Score field."""
        return base_field['type'] == BaseFieldType.CVSS_SCORE

    @classmethod
    def is_cvss_base_vector(cls, base_field: Dict) -> bool:
        """Check if field is CVSS Base Vector field."""
        return base_field['type'] == BaseFieldType.CVSS_VECTOR

    @classmethod
    def is_asset(cls, base_field: Dict) -> bool:
        """Check if field is Asset field."""
        return base_field['type'] == BaseFieldType.ASSET

    @classmethod
    def is_request(cls, base_field: Dict) -> bool:
        """Check if field is Request field."""
        return base_field['type'] == BaseFieldType.REQUEST

    @classmethod
    def is_datasource(cls, base_field: Dict) -> bool:
        """Check if field is Datasource field."""
        return base_field['type'] == BaseFieldType.DATASOURCE

    @classmethod
    def is_status(cls, base_field: Dict) -> bool:
        """Check if field is Status field."""
        return base_field['type'] == BaseFieldType.STATUS

    @classmethod
    def is_hostname(cls, base_field: Dict) -> bool:
        """Check if field is Hostname field."""
        return base_field['type'] == BaseFieldType.HOSTNAME

    @classmethod
    def is_uuid(cls, base_field: Dict) -> bool:
        """Check if field is UUID field."""
        return base_field['type'] == BaseFieldType.UUID

    @classmethod
    def is_ip(cls, base_field: Dict) -> bool:
        """Check if field is IP field."""
        return base_field['type'] == BaseFieldType.IP

    def build(self, field: Dict) -> ForwardRef('Field'):
        """Make from BaseField object advanced Field model."""
        final_type: FieldType = self.define_field_type(field)
        extra_fields = {
            'type': (Optional[FieldType], final_type),
            'value': (Optional[ValueType[final_type.name].value], None)
        }

        model = pydantic.create_model(
            FIELD_TYPE_MAP[final_type].__name__,
            **extra_fields,
            __base__=FIELD_TYPE_MAP[final_type]
        )
        return model

    def __call__(self, field: Dict) -> Any:
        """Make from BaseField object advanced Field model."""
        return self.build(field)



class HiveRestError(Exception):
    """Base exception for Hive rest client."""
    pass


class RestConnectionError(HiveRestError):
    """Exception for connection errors."""
    pass


class ServerUrlNotDefinedError(HiveRestError):
    """Exception for server URL not defined."""
    pass



from typing import Dict

from hivepy.schema_management.factories.schema_factory import SchemaFactory
from hivepy.template_management.models.issue_template import IssueTemplate


class IssueTemplateBuilder:
    """Issue template builder."""
    def __init__(self, schema_factory) -> None:
        self.schema_factory: SchemaFactory = SchemaFactory()

    def build(self, issue: Dict) -> IssueTemplate:
        """Build issue template."""
        issue['schema'] = self.schema_factory.make_issue_schema(issue['schema'])
        return IssueTemplate(**issue)



from typing import Dict

from hivepy.field_management.models.base_field import BaseField


class JsonFieldNormalizer:
    """Class that reshape raw field dictionary into BaseField dictionary."""
    @classmethod
    def build(cls, raw_field: Dict) -> Dict:
        """Build BaseField dictionary."""
        return BaseField(**raw_field).model_dump()

    def __call__(self, *args, **kwargs) -> Dict:
        """Call build."""
        return self.build(args[0] if args else kwargs)



from typing import Dict, Type, Optional, Any

import pydantic

from hivepy.project_management.project import Project
from hivepy.schema_management.factories.schema_factory import SchemaFactory
from hivepy.schema_management.models.project_schema import ProjectSchema


class ProjectBuilder:
    """Project builder class."""
    def __init__(self, schema_factory=None) -> None:
        """Initialize the project builder with a template builder."""
        self.schema_factory: SchemaFactory = schema_factory if schema_factory \
            else self.create_schema_factory()

    @staticmethod
    def create_schema_factory() -> SchemaFactory:
        """Create a project schema builder."""
        return SchemaFactory()

    def build(self, project_data: Dict) -> Project:
        """Build a project from the given project data."""
        data: Dict = project_data.pop('data', {})
        schema: Dict = project_data.pop('projectSchema', {})

        if schema and data:
            project_model = self.create_project_model(schema)
            return project_model(**data | project_data)
        else:
            return Project(**project_data)

    def create_project_model(self, schema: Dict) -> Type['Project']:
        """Create a project model from the given project schema."""
        project_schema: ProjectSchema = self.schema_factory.make_project_schema(schema)
        fields = project_schema.predefined_fields_settings + project_schema.additional_fields_settings
        schema_fields = {field.name: (Optional[Any], None) for field in fields}
        return pydantic.create_model('Project', **schema_fields, __base__=Project)



from typing import Dict, Union

from hivepy.api_management.enums import ObjectType
from hivepy.issue_management.issue import Issue
from hivepy.issue_management.issue_builder import IssueBuilder
from hivepy.project_management.project import Project
from hivepy.project_management.project_builder import ProjectBuilder
from hivepy.schema_management.factories import SchemaFactory
from hivepy.schema_management.models.issue_schema import IssueSchema


class Builder:
    """Class into which other object builders injected."""
    def __init__(self) -> None:
        """Initialize builder."""
        self.schema_factory: SchemaFactory = SchemaFactory()
        self.project_builder: ProjectBuilder = ProjectBuilder(self.schema_factory)
        self.issue_builder: IssueBuilder = IssueBuilder(self.schema_factory)

    def build_project(self, project_data: dict) -> Project:
        """Build project."""
        return self.project_builder.build(project_data)

    def build_issue(self, issue_schema: IssueSchema, issue_data: Dict) -> Issue:
        """Build issue."""
        return self.issue_builder.build(issue_schema, issue_data)

    def build(self, object_type: ObjectType, *args, **kwargs):
        """Build object."""
        if object_type == ObjectType.PROJECT:
            return self.build_project(*args, **kwargs)
        if object_type == ObjectType.ISSUE:
            return self.build_issue(*args, **kwargs)

    def get_project_builder(self) -> ProjectBuilder:
        """Get project builder."""
        return self.project_builder

    def get_issue_builder(self) -> IssueBuilder:
        """Get issue builder."""
        return self.issue_builder

    def __call__(self, *args, **kwargs) -> Union[Issue, Project]:
        """Call."""
        return self.build(*args, **kwargs)



from hivepy.field_management.models.base_field import BaseField
from hivepy.field_management.models.fields import (Text,
                                                   SingleTextSuggested,
                                                   MultiTextSuggested,
                                                   Checkbox,
                                                   Checkboxes,
                                                   RadioButton,
                                                   Select,
                                                   MultiSelect,
                                                   Link,
                                                   TextMarkdown,
                                                   Image)


__all__ = [
    'BaseField',
    'Text',
    'SingleTextSuggested',
    'MultiTextSuggested',
    'Checkbox',
    'Checkboxes',
    'RadioButton',
    'Select',
    'MultiSelect',
    'Link',
    'TextMarkdown',
    'Image',
]



from hivepy.template_management.builders.project_template_builder import ProjectTemplateBuilder
from hivepy.template_management.builders.issue_template_builder import IssueTemplateBuilder
from hivepy.template_management.builders.template_builder import TemplateBuilder


__all__ = [
    'ProjectTemplateBuilder',
    'IssueTemplateBuilder',
    'TemplateBuilder',
]



from hivepy.rest.state import State
from hivepy.rest import exceptions, http_client
from hivepy.rest.http_client.http_client import HTTPClient

__all__ = [
    'HTTPClient',
    'State',
    'exceptions',
    'http_client',
]



from hivepy.field_management.builder.field_builder import FieldBuilder
from hivepy.field_management.builder.field_model_builder import FieldModelBuilder


__all__ = [
    'FieldBuilder',
    'FieldModelBuilder',
]



from typing import Optional

from hivepy.field_management.builder.field_builder import FieldBuilder
from hivepy.schema_management.models.issue_schema import IssueSchema


class IssueSchemaBuilder:
    """Class that builds an IssueSchema object from a raw issue schema dictionary."""
    def __init__(self, field_builder=None):
        self.field_builder: Optional[FieldBuilder] = field_builder

    def build(self, issue_schema: dict) -> IssueSchema:
        """Build ProjectSchema object from raw data."""
        data = issue_schema.copy()
        raw_additional_fields = data.pop('additionalFieldsSettings', [])
        raw_predefined_fields = data.pop('predefinedFieldsSettings', [])
        data['additionalFieldsSettings'] = [self.field_builder.build(field) for field in raw_additional_fields]
        data['predefinedFieldsSettings'] = [self.field_builder.build(field) for field in raw_predefined_fields]
        return IssueSchema(**data)

    def __call__(self, project_schema: dict) -> IssueSchema:
        """Build ProjectSchema object from raw data."""
        return self.build(project_schema)



import datetime
from enum import Enum, StrEnum


class BaseFieldType(StrEnum):
    """Enumeration of base field types."""
    TEXT = "TEXT"
    TEXT_SUGGESTED = "TEXT_SUGGESTED"
    TEXT_MARKDOWN = "TEXT_MD"
    INTEGER = "INTEGER"
    BOOLEAN = "BOOLEAN"
    FLOAT = "FLOAT"
    FILE = "FILE"
    DATE = "DATE"
    DATETIME = "DATETIME"
    IMAGE = "IMAGE"
    LINK = "LINK"
    UUID = "UUID"
    IP = "IP"
    HOSTNAME = "HOSTNAME"
    ASSET = "ASSET"
    REQUEST = "REQUEST"
    DATASOURCE = "DATASOURCE"
    STATUS = "STATUS"
    CVSS_SCORE = "CVSS_BASE_SCORE"
    CVSS_VECTOR = "CVSS_BASE_VECTOR"


class FieldType(StrEnum):
    """Enumeration of field types."""
    FLOAT = "FLOAT"
    INTEGER = "INTEGER"
    TEXT = "TEXT"
    TEXT_MARKDOWN = "TEXT_MD"
    SINGLE_TEXT_SUGGESTED = 'SINGLE_TEXT_SUGGESTED'
    MULTI_TEXT_SUGGESTED = "MULTI_TEXT_SUGGESTED"
    RADIOBUTTON = "RADIOBUTTON"
    CHECKBOX = "CHECKBOX"
    CHECKBOXES = "CHECKBOXES"
    SELECT = "SELECT"
    MULTISELECT = "MULTISELECT"
    DATE = "DATE"
    DATETIME = "DATETIME"
    LINK = "LINK"
    IMAGE = "IMAGE"
    FILE = "FILE"
    UUID = "UUID"
    IP = "IP"
    HOSTNAME = "HOSTNAME"
    ASSET = "ASSET"
    REQUEST = "REQUEST"
    DATASOURCE = "DATASOURCE"
    STATUS = "STATUS"
    CVSS_SCORE = "CVSS_BASE_SCORE"
    CVSS_VECTOR = "CVSS_BASE_VECTOR"


class ValueType(Enum):
    """Enumeration of value types for different fields."""
    INTEGER = int
    FLOAT = float
    TEXT = str
    UUID = str
    STATUS = str
    TEXT_MARKDOWN = str
    SINGLE_TEXT_SUGGESTED = str
    MULTI_TEXT_SUGGESTED = list
    HOSTNAME = list
    IP = list
    ASSET = list
    REQUEST = list
    DATASOURCE = str
    CVSS_SCORE = int
    CVSS_VECTOR = str
    CHECKBOX = bool
    RADIOBUTTON = str
    CHECKBOXES = list
    SELECT = str
    MULTISELECT = list
    DATE = datetime.date
    DATETIME = datetime.datetime
    LINK = str
    IMAGE = str
    FILE = bytes



import uuid
from datetime import datetime
from enum import StrEnum
from typing import Optional, List, Any

import pydantic

from hivepy.schema_management.models.base_schema import BaseSchema

asset_version = StrEnum('AssetVersion', ['COMPLEX', 'SIMPLE'])
score_method = StrEnum('ScoreMethod', ['COMPLEX', 'SIMPLE'])


class IssueSchema(BaseSchema):
    id: uuid.UUID
    name: str
    description: Optional[str] = pydantic.Field(default=None)
    is_default: bool = pydantic.Field(default=False, alias='isDefault')
    predefined_fields_settings: Optional[List[Any]] = pydantic.Field(default=None, alias='predefinedFieldsSettings')
    additional_fields_settings: Optional[List[Any]] = pydantic.Field(default=None, alias='additionalFieldsSettings')
    asset_version: str = pydantic.Field(default='SIMPLE', alias='assetVersion')
    score_method: str = pydantic.Field(default='SIMPLE', alias='scoreMethod')
    structures: Optional[List[Any]] = pydantic.Field(default=None)
    fields_order: List = pydantic.Field(alias='order')
    timestamp: Optional[datetime] = pydantic.Field(default=None)
    user_id: Optional[uuid.UUID] = pydantic.Field(default=None, alias='userId')



from typing import Dict, Type, Optional, Any

import pydantic

from hivepy.issue_management.issue import Issue
from hivepy.schema_management.factories.schema_factory import SchemaFactory
from hivepy.schema_management.models.issue_schema import IssueSchema


class IssueBuilder:
    def __init__(self, schema_factory=None) -> None:
        """Initialize the project builder with a template builder."""
        self.schema_factory: SchemaFactory = schema_factory if schema_factory \
            else self.create_schema_factory()

    @staticmethod
    def create_schema_factory() -> SchemaFactory:
        """Create a project schema builder."""
        return SchemaFactory()

    def build(self, issue_schema: IssueSchema,  issue_data: Dict) -> Issue:
        """Build a project from the given project data."""
        issue_model = self.create_issue_model(issue_schema)
        return issue_model(**issue_data)

    @staticmethod
    def create_issue_model(schema: IssueSchema) -> Type['Issue']:
        """Create a project model from the given project schema."""
        fields = schema.predefined_fields_settings + schema.additional_fields_settings
        schema_fields = {field.name: (Optional[Any], None) for field in fields}
        return pydantic.create_model('Issue', **schema_fields, __base__=Issue)



from typing import List, Optional, Any, Dict, Union
from hivepy.field_management.enums import BaseFieldType
import pydantic


class BaseField(pydantic.BaseModel):
    display_name: str = pydantic.Field(alias='displayName')
    name: str
    type: Optional[BaseFieldType] = pydantic.Field(default=None)
    value: Optional[Any] = pydantic.Field(default=None)
    allowed_values: Optional[List[str]] = pydantic.Field(default=list(), alias='allowedValues')
    is_list: bool = pydantic.Field(default=False, alias='isList')
    required: bool = pydantic.Field(default=False)
    hidden: bool = pydantic.Field(default=False)
    initially_collapsed: Optional[bool] = pydantic.Field(default=None, alias='initiallyCollapsed')
    select: Optional[bool] = pydantic.Field(default=None)
    placeholder: Optional[str] = pydantic.Field(default=None)
    hint: Optional[str] = pydantic.Field(default=None)
    placeholder_link_text: Optional[str] = pydantic.Field(default=None)
    placeholder_link_url: Optional[str] = pydantic.Field(default=None)

    model_config = pydantic.ConfigDict(
        populate_by_name=True
    )

    def __str__(self) -> str:
        """Return string representation of object."""
        return f'{self.__class__.__qualname__}({", ".join(f"{key}={value}" for key, value in self.dict().items())})'

    def __call__(self) -> Any:
        """Return value of the field."""
        return self.value

    @pydantic.field_validator('allowed_values', mode='before')
    def flatten_allowed_values(cls, v: Optional[Union[List[Dict], List[str]]]) -> Optional[List[str]]:
        """Flatten allowed values."""
        if not v:
            return []

        if hasattr(v[0], 'values'):
            return [x['displayValue'] for x in v]

        return v

    @pydantic.model_validator(mode='before')
    def parse_metadata(cls, v: Dict) -> Dict:
        """Parse metadata."""
        if not (metadata := v.pop('metadata', {})):
            return v

        [v.update({key: val}) for key, val in metadata.items()]
        return v



import uuid
from typing import Optional, List

import pydantic

from hivepy.template_management.models.base_template import BaseTemplate
from hivepy.schema_management.models.project_schema import ProjectSchema


class ProjectTemplate(BaseTemplate):
    """Project template model."""
    id: uuid.UUID
    name: str
    description: Optional[str] = pydantic.Field(default=None)
    is_default: Optional[bool] = pydantic.Field(default=None, alias='isDefault')
    default_issue_schema_id: Optional[uuid.UUID] = pydantic.Field(default=None, alias='defaultIssueSchemaID')
    default_report_template_name: Optional[str] = pydantic.Field(default=None, alias='defaultReportTemplateName')
    schema: Optional[ProjectSchema] = pydantic.Field(default=None, alias='projectSchema')
    checklists: Optional[List] = pydantic.Field(default=None)
    wiki_pages: Optional[List] = pydantic.Field(default=None, alias='wikiPages')



from typing import Type, Optional, Callable, ForwardRef

from hivepy.api_management.enums import ClientMode, ObjectType

HiveApi = ForwardRef('HiveApi')


class ModeHandler:
    """
    Class wrapper for HiveApi.
    If mode is set to JSON, response will be returned as is.
    If mode is set to RAW_ITEMS, response will be returned as list of items or raw item if only one.
    If mode is set to OBJECT, response will be deserialized to object using provided ObjectType.
    """
    def __init__(self, hive_cls) -> None:
        """Initialize mode handler."""
        self._hive_cls: Type['HiveApi'] = hive_cls
        self._instance: Optional['HiveApi'] = None

    def __call__(self, *args, **kwargs) -> HiveApi:
        """Call wrapped class."""
        if not self._instance:
            self._instance = self._hive_cls(*args, **kwargs)
            self._builder = self._instance.builder
        return self._instance

    def __getattr__(self, name: str) -> HiveApi:
        """Get attribute from wrapped class."""
        return getattr(self._instance, name)

    @staticmethod
    def deserialize(to: ObjectType) -> Callable:
        """Deserialize response to object."""
        def decorator(method) -> Callable:
            """Decorator."""
            def wrapper(self, *args, **kwargs):
                """Wrapper."""
                result = method(self, *args, **kwargs)
                if self.mode == ClientMode.JSON:
                    return result

                if hasattr(result, 'get'):  # Response is a dict
                    result = result.get('items', result)    # Get items from Dict
                if self.mode == ClientMode.RAW_ITEMS:
                    return result  # Return list of items or raw item if only one

                if self.mode == ClientMode.OBJECT:
                    if hasattr(result, 'append'):
                        return [self._builder.build(to, item) for item in result]
                    return self.builder.build(to, result)
            return wrapper
        return decorator



from enum import StrEnum, auto
from typing import Dict, Self, Union, List

import requests.utils
from requests import Session, Response

from hivepy.rest.http_client.exceptions import *


class HTTPMethod(StrEnum):
    """Enumeration of HTTP methods."""
    GET = auto()
    PATCH = auto()
    POST = auto()
    PUT = auto()
    DELETE = auto()

    def __str__(self) -> str:
        """Return string representation of HTTP method."""
        return self.value.upper()


class HTTPClient:
    """Base implementation of the HTTP client."""
    session: Session = Session()

    def _send(self, method: HTTPMethod, *args, **kwargs) -> Union[Dict, bytes]:
        """Universal method for sending requests."""
        try:
            response: Response = self.session.request(method, *args, **kwargs)
            try:
                return response.json()
            except requests.JSONDecodeError:
                return response.content
        except requests.ConnectionError as e:
            if 'SOCKSHTTPSConnectionPool' in str(e):
                proxy = self.session.proxies.get('https')
                raise SocksProxyError(f'Couldn\'t connect via "{proxy}". Check it.')
            else:
                raise ClientConnectionError(e)

    def _update_params(self, **kwargs) -> Self:
        """Update session parameters if they are not None and exist in session."""
        [setattr(self.session, key, value) for key, value in kwargs.items()
         if value is not None and hasattr(self.session, key)]
        return self

    def get(self, *args, **kwargs) -> Union[Dict, List, bytes]:
        """Send GET request."""
        return self._send(HTTPMethod.GET, *args, **kwargs)

    def post(self, *args, **kwargs) -> Union[Dict, List, bytes]:
        """Send POST request."""
        return self._send(HTTPMethod.POST, *args, **kwargs)

    def put(self, *args, **kwargs) -> Dict:
        """Send PUT request."""
        return self._send(HTTPMethod.PUT, *args, **kwargs)

    def patch(self, *args, **kwargs) -> Dict:
        """Send PATCH request."""
        return self._send(HTTPMethod.PATCH, *args, **kwargs)

    def delete(self, *args, **kwargs) -> Dict:
        """Send DELETE request."""
        return self._send(HTTPMethod.DELETE, *args, **kwargs)

    def add_headers(self, headers: Dict) -> Self:
        """Method injects headers into session."""
        self.session.headers.update(headers)
        return self

    def update_params(self, **kwargs) -> Self:
        """Method updates session parameters."""
        self._update_params(**kwargs)
        return self

    @property
    def params(self):
        """Method to get session parameters."""
        return self.session.__dict__



from hivepy.schema_management.builders.issue_schema_builder import IssueSchemaBuilder
from hivepy.schema_management.builders.schema_builder import SchemaBuilder
from hivepy.schema_management.builders.project_schema_builder import ProjectSchemaBuilder


__all__ = [
    "IssueSchemaBuilder",
    "SchemaBuilder",
    "ProjectSchemaBuilder",
]



import functools
from typing import Dict, Union

from hivepy.field_management.builder.field_builder import FieldBuilder
from hivepy.schema_management.builders.issue_schema_builder import IssueSchemaBuilder
from hivepy.schema_management.builders.project_schema_builder import ProjectSchemaBuilder
from hivepy.schema_management.models import IssueSchema, ProjectSchema


class SchemaBuilder:
    """Class that builds schema objects."""
    field_builder: FieldBuilder = FieldBuilder()
    issue_schema_builder: IssueSchemaBuilder = IssueSchemaBuilder(field_builder)
    project_schema_builder: ProjectSchemaBuilder = ProjectSchemaBuilder(field_builder)

    @classmethod
    def build(cls, schema_type: str, schema: Dict) -> Union[IssueSchema, ProjectSchema]:
        """Build schema object from raw data."""
        if schema_type == 'issue':
            return cls.issue_schema_builder(schema)
        elif schema_type == 'project':
            return cls.project_schema_builder(schema)
        else:
            raise ValueError(f"Schema type {schema_type} not supported.")

    @functools.partialmethod
    def build_issue_schema(self, schema: Dict) -> IssueSchema:
        """Build IssueSchema object from raw data."""
        return self.build('issue', schema)

    @functools.partialmethod
    def build_project_schema(self, schema: Dict) -> ProjectSchema:
        """Build ProjectSchema object from raw data."""
        return self.build('project', schema)

    def __call__(self, schema_type: str, schema: Dict) -> Union[IssueSchema, ProjectSchema]:
        """Build schema object from raw data."""
        return self.build(schema_type, schema)



from hivepy.template_management.models.project_template import ProjectTemplate
from hivepy.template_management.models.issue_template import IssueTemplate


__all__ = [
    'IssueTemplate',
    'ProjectTemplate',
]



from typing import Dict, ForwardRef

from hivepy.field_management.builder.field_model_builder import FieldModelBuilder
from hivepy.field_management.json_field_normalizer import JsonFieldNormalizer


class FieldBuilder:
    """Class that builds a Field object from a raw field dictionary."""
    def __init__(self) -> None:
        """Initialize FieldBuilder class."""
        self.field_model_builder = FieldModelBuilder()
        self.json_field_normalizer = JsonFieldNormalizer()

    def build(self, field: Dict) -> ForwardRef('Field'):
        """Make from BaseField object advanced Field model."""
        field = self.json_field_normalizer(field)
        model = self.field_model_builder(field)
        return model(**field)

    def __call__(self, field: Dict) -> ForwardRef('Field'):
        """Make from BaseField object advanced Field model."""
        return self.build(field)



from setuptools import setup, find_packages

setup(
   name='hw-hivepy',
   version='0.3',
   description='Small wrapper for HexWay Hive API',
   long_description=open('README.md').read(),
   long_description_content_type='text/markdown',
   include_package_data=True,
   author='@Cur1iosity',
   author_email='Cur1iosity@protonmail.com',
   url='https://github.com/Cur1iosity/hivepy',
   packages=find_packages(),
   install_requires=['requests>=2.31.0', 'pydantic>=2.4.0'],
   extras_require={
      'dev': [
         'pytest',
      ],
   },
   classifiers=[
      'Programming Language :: Python :: 3',
      'License :: OSI Approved :: MIT License',
      'Operating System :: OS Independent',
   ],
   python_requires='>=3.12',
)



import pydantic


class BaseTemplate(pydantic.BaseModel):
    ...

    model_config = pydantic.ConfigDict(
        populate_by_name=True
    )

    def __str__(self) -> str:
        """Return string representation of object."""
        return f'{self.__class__.__qualname__}({", ".join(f"{key}={value}" for key, value in self.dict().items())})'



__description__ = "Small wrapper for HexWay Hive API "
__author__ = "@Cur1iosity"
__email__ = "Cur1iosity@protonmail.com"
__licence__ = "MIT"

from hivepy import rest
from hivepy.hive_api import HiveApi


__all__ = [
    'rest',
    'HiveApi',
]



import uuid
from datetime import datetime
from typing import Dict, List, Optional

import pydantic


class Issue(pydantic.BaseModel):
    id: Optional[uuid.UUID] = pydantic.Field(alias='uuid')
    internal_id: Optional[int] = pydantic.Field(default=None, alias='id')
    name: str
    labels: Optional[List[str]] = pydantic.Field(default=None)

    ips: Optional[List[str]] = pydantic.Field(default=None)
    hostnames: Optional[List[str]] = pydantic.Field(default=None)

    status: str
    sync_status: Optional[str] = pydantic.Field(default=None, alias='syncStatus')

    creator: Dict = pydantic.Field(alias='nodeCreator')

    node_id: Optional[str] = pydantic.Field(default=None, alias='nodeId')
    parent_id: Optional[str] = pydantic.Field(default=None, alias='parentId')

    edit_time: datetime = pydantic.Field(default=None, alias='editTime')
    post_time: datetime = pydantic.Field(default=None, alias='postTime')

    checkmarks: Optional[List] = pydantic.Field(default=None)
    assets: Optional[List] = pydantic.Field(default=None)

    issue_source_type: Optional[str] = pydantic.Field(default=None, alias='issueSourceType')
    weakness_type: Optional[str] = pydantic.Field(default=None, alias='weaknessType')

    cvss_vector: Optional[str] = pydantic.Field(default=None, alias='cvssVector')
    cvss_score: Optional[float] = pydantic.Field(default=None, alias='cvssScore')

    probability_score: int = pydantic.Field(default=None, alias='probabilityScore')
    criticality_score: int = pydantic.Field(default=None, alias='criticalityScore')

    requests: Optional[List[Dict]] = pydantic.Field(default=None)
    notes: Optional[List] = pydantic.Field(default=None)
    files: Optional[List[Dict]] = pydantic.Field(default=None)

    general_description: Optional[str] = pydantic.Field(default=None, alias='generalDescription')
    recommendations: Optional[str] = pydantic.Field(default=None)
    reproduce_description: Optional[str] = pydantic.Field(default=None, alias='reproduceDescription')
    risk_description: Optional[str] = pydantic.Field(default=None, alias='riskDescription')
    technical_description: Optional[str] = pydantic.Field(default=None, alias='technicalDescription')

    model_config = pydantic.ConfigDict(
        populate_by_name=True
    )

    # @pydantic.field_validator('post_time', 'edit_time', mode='before')
    # def validate_datetime(cls, value: str) -> str:
    #     """Validate datetime field."""
    #     try:
    #         dt: datetime = datetime.strptime(value, '%Y-%m-%dT%H:%M:%S.%fZ')
    #     except ValueError:
    #         raise ValueError('Invalid datetime.')
    #     return datetime.strftime(dt, '%d-%m-%Y %H:%M:%S')

    @pydantic.model_validator(mode='before')
    def make_additional_fields_flat(cls, value: Dict) -> Dict:
        """Make additional fields flat."""
        if not (additional_fields := value.get('additionalFields', {})):
            return value
        value.update(additional_fields)
        return value

    def __str__(self) -> str:
        """Return string representation of object."""
        return f'{self.__class__.__name__}({"".join(f"{key}={value}, " for key, value in self.model_dump().items())})'



from hivepy.template_management import builders, models
from hivepy.template_management.builders.template_builder import TemplateBuilder


__all__ = [
    'builders',
    'models',
    'TemplateBuilder',
]



from hivepy.field_management.json_field_normalizer import JsonFieldNormalizer
from hivepy.field_management import models, enums
from hivepy.field_management.models.fields import FIELD_TYPE_MAP
from hivepy.field_management.builder.field_builder import FieldBuilder


__all__ = [
    "JsonFieldNormalizer",
    "models",
    "enums",
    "FIELD_TYPE_MAP",
    "FieldBuilder",
]



from enum import StrEnum, auto


class ClientMode(StrEnum):
    """Enumeration of client modes."""
    JSON = auto()
    RAW_ITEMS = auto()
    OBJECT = auto()


class ObjectType(StrEnum):
    ISSUE = auto()
    PROJECT = auto()
    USER = auto()



import datetime
import uuid
from typing import Optional, List, Any

import pydantic

from hivepy.schema_management.models.base_schema import BaseSchema


class ProjectSchema(BaseSchema):
    """Project schema model."""
    id: uuid.UUID
    predefined_fields_settings: Optional[List[Any]] = pydantic.Field(default=None, alias='predefinedFieldsSettings')
    additional_fields_settings: Optional[List[Any]] = pydantic.Field(default=None, alias='additionalFieldsSettings')
    fields_order: List = pydantic.Field(alias='fieldsOrder')

    created: datetime.datetime
    updated: Optional[datetime.datetime] = pydantic.Field(default=None)
    version: int
